from langchain.tools import tool
import traceback

try:
    from .utils.db import *
    from .llm import get_model
    from .top_bar_wrapper import wrapper
    from .llm_settings import llm_settings
except ImportError:
    from utils.db import *
    from top_bar_wrapper import wrapper
    from llm_settings import llm_settings


@wrapper
def extract_possible_coordinates_of_text_(text:str) -> dict:
    """
    A function to extract possible coordinates of a text on the screen.
    """


    import pyautogui
    import pytesseract
    from PIL import ImageGrab
    import cv2
    import numpy as np





    def capture_screen():
        # Capture the entire screen
        screenshot = ImageGrab.grab()
        screenshot_np = np.array(screenshot)
        return cv2.cvtColor(screenshot_np, cv2.COLOR_RGB2BGR)

    def find_all_text_coordinates(confidence=70):
        # Capture the screen and convert it to grayscale
        img = capture_screen()
        gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)

        # Use pytesseract to get data about the text on the screen
        data = pytesseract.image_to_data(gray, output_type=pytesseract.Output.DICT)

        texts_with_coordinates = []
        for i in range(len(data['text'])):
            if data['text'][i].strip() and int(data['conf'][i]) > confidence:
                x, y, w, h = data['left'][i], data['top'][i], data['width'][i], data['height'][i]
                texts_with_coordinates.append({
                    'text': data['text'][i],
                    'coordinates': (x + w // 2, y + h // 2)
                })
        
        return texts_with_coordinates


    # return a list of possible coordinates of the text
    texts_with_coordinates = find_all_text_coordinates()
    result =  [{"name": item["text"], "coordinates": {"x":item["coordinates"][0], "y":item["coordinates"][1]}} for item in texts_with_coordinates if item["text"].startswith(text)]
    return result


extract_possible_coordinates_of_text = tool(extract_possible_coordinates_of_text_)



@wrapper
def click_on_a_text_on_the_screen_(text: str, click_type: str = "singular") -> bool:
    """
    A function to click on a text on the screen.

    Parameters:
    - text (str): The text to be clicked on.
    - click_type (str): The type of click to be performed. The default value is "singular". Possible values are "singular" and "double".

    Returns:
    - bool: True if the text was clicked on successfully, False otherwise.
    """
    try:
        import pyautogui
        from PIL import ImageGrab, ImageDraw

        pyautogui.FAILSAFE = False

        from interpreter import OpenInterpreter

        interpreter = OpenInterpreter()

        model = load_model_settings()
        if llm_settings[model]["provider"] == "azureai":
            interpreter.llm.model = f"azure/{model}"
            import os 
            os.environ["AZURE_API_KEY"] = load_api_key()
            os.environ["AZURE_API_BASE"] = load_openai_url()
            os.environ["AZURE_API_VERSION"] = load_api_version()

        else:
            interpreter.llm.api_key = load_api_key()


        text_locations = extract_possible_coordinates_of_text_(text)

        x, y = text_locations[0]["coordinates"]["x"], text_locations[0]["coordinates"]["y"]
 


        screen_width, screen_height = pyautogui.size()
        screenshot = ImageGrab.grab()
        screenshot_width, screenshot_height = screenshot.size

        # Calculate scaling factors
        scale_x = screen_width / screenshot_width
        scale_y = screen_height / screenshot_height

        # Apply scaling factors to coordinates
        scaled_x = x * scale_x
        scaled_y = y * scale_y

        if click_type == "singular":
            interpreter.computer.mouse.click(x=scaled_x, y=scaled_y)
        elif click_type == "double":
            interpreter.computer.mouse.double_click(x=scaled_x, y=scaled_y)
        return True
    except:
        traceback.print_exc()
        return False


click_on_a_text_on_the_screen = tool(click_on_a_text_on_the_screen_)


@wrapper
def move_on_a_text_on_the_screen_(text: str) -> bool:
    """
    A function to move on a text on the screen.

    Parameters:
    - text (str): The text to be moved on.

    Returns:
    - bool: True if the text was moved on successfully, False otherwise.
    """
    try:
        import pyautogui

        pyautogui.FAILSAFE = False

        from interpreter import OpenInterpreter

        interpreter = OpenInterpreter()

        model = load_model_settings()
        if llm_settings[model]["provider"] == "azureai":
            interpreter.llm.model = f"azure/{model}"
            import os 
            os.environ["AZURE_API_KEY"] = load_api_key()
            os.environ["AZURE_API_BASE"] = load_openai_url()
            os.environ["AZURE_API_VERSION"] = load_api_version()

        else:
            interpreter.llm.api_key = load_api_key()

        screenshot = pyautogui.screenshot()

        text_locations = interpreter.computer.display.find_text(
            text, screenshot=screenshot
        )

        print(text_locations)

        x, y = text_locations[0]["coordinates"]
        x *= interpreter.computer.display.width
        y *= interpreter.computer.display.height
        x = int(x)
        y = int(y)

        interpreter.computer.mouse.move(x=x, y=y, screenshot=screenshot)

        return True
    except:
        traceback.print_exc()
        return False


move_on_a_text_on_the_screen = tool(move_on_a_text_on_the_screen_)


@wrapper
def click_on_a_icon_on_the_screen_(
    icon_name: str, click_type: str = "singular"
) -> bool:
    """
    A function to click on a icon name on the screen.

    Parameters:
    - icon_name (str): The icon name to be clicked on.
    - click_type (str): The type of click to be performed. The default value is "singular". Possible values are "singular" and "double".

    Returns:
    - bool: True if the icon name was clicked on successfully, False otherwise.
    """
    try:
        import pyautogui

        pyautogui.FAILSAFE = False

        from interpreter import OpenInterpreter

        screenshot = pyautogui.screenshot()

        interpreter = OpenInterpreter()

        model = load_model_settings()
        if llm_settings[model]["provider"] == "azureai":
            interpreter.llm.model = f"azure/{model}"
            import os 
            os.environ["AZURE_API_KEY"] = load_api_key()
            os.environ["AZURE_API_BASE"] = load_openai_url()
            os.environ["AZURE_API_VERSION"] = load_api_version()

        else:
            interpreter.llm.api_key = load_api_key()

        if click_type == "singular":
            interpreter.computer.mouse.click(icon=icon_name, screenshot=screenshot)
        elif click_type == "double":
            interpreter.computer.mouse.double_click(
                icon=icon_name, screenshot=screenshot
            )
        return True

    except:
        traceback.print_exc()
        return False


click_on_a_icon_on_the_screen = tool(click_on_a_icon_on_the_screen_)


@wrapper
def move_on_a_icon_on_the_screen_(
    icon_name: str,
) -> bool:
    """
    A function to move on a icon name on the screen.

    Parameters:
    - icon_name (str): The icon name to be move on.

    Returns:
    - bool: True if the icon name was moved on successfully, False otherwise.
    """
    try:
        import pyautogui

        pyautogui.FAILSAFE = False

        from interpreter import OpenInterpreter

        screenshot = pyautogui.screenshot()

        interpreter = OpenInterpreter()

        model = load_model_settings()
        if llm_settings[model]["provider"] == "azureai":
            interpreter.llm.model = f"azure/{model}"
            import os 
            os.environ["AZURE_API_KEY"] = load_api_key()
            os.environ["AZURE_API_BASE"] = load_openai_url()
            os.environ["AZURE_API_VERSION"] = load_api_version()

        else:
            interpreter.llm.api_key = load_api_key()

        interpreter.computer.mouse.move(icon=icon_name, screenshot=screenshot)
        return True

    except:
        traceback.print_exc()
        return False


move_on_a_icon_on_the_screen = tool(move_on_a_icon_on_the_screen_)


def mouse_scroll_(direction: str, amount: int = 1) -> bool:
    """
    A function to scroll the mouse wheel.

    Parameters:
    - direction (str): The direction of the scroll. Possible values are "up" and "down".
    - amount (int): The amount of scrolling to be performed. The default value is 1.

    Returns:
    - bool: True if the scrolling was performed successfully, False otherwise.
    """
    try:
        import pyautogui

        pyautogui.FAILSAFE = False

        if direction == "up":
            pyautogui.scroll(amount)
        elif direction == "down":
            pyautogui.scroll(-amount)
        return True
    except:
        traceback.print_exc()
        return False


mouse_scroll = tool(mouse_scroll_)


@wrapper
def get_texts_on_the_screen_() -> str:
    """
    It returns the texts on the screen.
    """

    try:
        pass

    except:
        pass

    import pyautogui

    the_screenshot_path = "temp_screenshot.png"
    the_screenshot = pyautogui.screenshot()
    the_screenshot.save(the_screenshot_path)

    from interpreter.core.computer.utils.computer_vision import pytesseract_get_text

    return pytesseract_get_text(the_screenshot_path)


get_texts_on_the_screen = tool(get_texts_on_the_screen_)



try:
    from .start import start

    from .agentic import Agent

    from .tooler import Tool
except:
    pass
__version__ = '0.23.20'  # fmt: skip


import os
import time
import subprocess
import requests


from requests.packages.urllib3.exceptions import InsecureRequestWarning

requests.packages.urllib3.disable_warnings(InsecureRequestWarning)



class instance:
    def __init__(self, url):
        self.url = url


    def request(self):
        pass





class interface:
    pass



class local_instance(instance):
    def __init__(self):
        super().__init__("http://localhost:7541")
        from .remote import Remote_Client

        self.client = Remote_Client(self.url)

    def request(self, the_request, the_response):

        return self.client.request(the_request, the_response)


    def start(self):
        command = "python -c 'from gpt_computer_assistant import start; start(True);'"
        self.process = subprocess.Popen(command, shell=True)


    def close(self):
        try:
            self.client.stop_server()
        except:
            pass

        self.process.terminate()
        self.process.wait()



    def client_status(self):
        return self.client.status




class local(interface):

    @staticmethod
    def instance():
        the_instance = local_instance()
        the_instance.start()

        time.sleep(5)

        client_status = the_instance.client_status()

        if not client_status:
            raise Exception("Failed to start the local instance")
        
        return the_instance
        


class cloud_instance(instance):
    def __init__(self):
        super().__init__("https://free_cloud_1.gca.dev/")


    def request(self, the_request, the_response):
        response = requests.post(self.url+"request", data={"request": the_request, "response": the_response, "instance":self.instance_id}, verify=False)
        try:
            json_response = response.json()
            return json_response["result"]
        except:
            return response.text
        


    def current_screenshot(self):
        response = requests.post(self.url+"screenshot_instance", data={"instance":self.instance_id}, verify=False)

        its_an_error = False

        try:
            the_json = response.json()
            if "result" in the_json:
                its_an_error = True
        except:
            pass


        if not its_an_error:
            with open('current_screenshot.png', 'wb') as file:
                file.write(response.content)
            import matplotlib.pyplot as plt
            import matplotlib.image as mpimg

            img = mpimg.imread('current_screenshot.png')
            plt.imshow(img)
            plt.axis('off')
            plt.show()





    def start(self):
        req = requests.get(self.url+"start_instance", verify=False)
        the_json = req.json()

        self.instance_id = the_json["result"]


    def close(self):
        req = requests.post(self.url+"stop_instance", data={"instance": self.instance_id}, verify=False)
        the_json = req.json()
        return the_json["result"]

    def client_status(self):
        return True



class cloud(interface):

    @staticmethod
    def instance():
        the_instance = cloud_instance()
        the_instance.start()

        return the_instance


from langchain.tools import tool

try:
    from .utils.db import load_api_key
    from .llm import get_model
    from .top_bar_wrapper import wrapper
    from .agent.agent_tools import get_tools
except ImportError:
    from llm import get_model
    from top_bar_wrapper import wrapper
    from agent.agent_tools import get_tools


@wrapper
def search_on_internet_and_report_team_(
    the_subject: str, copy_to_clipboard: bool = False
) -> str:
    """
    A function to search the internet generates a report. Just use in detailed searches

    Parameters:
    - the_subject (str): The subject to search the internet for.
    - copy_to_clipboard (bool): A flag to indicate whether to copy the report to the clipboard. The default value is False.

    Returns:
    - str: The report of the search.
    """

    from crewai import Task, Crew, Agent

    tools = get_tools()

    the_tool_list = []
    for each in tools:
        if "team" not in each.name:
            the_tool_list.append(each)

    # Create the agents

    search_engine_master = Agent(
        role="search_engine_master",
        goal="To meticulously comb through the vast expanse of the internet, utilizing advanced search algorithms and techniques to find the most relevant, accurate, and up-to-date information on the given subject.",
        backstory="Born from the digital ether, I am the search engine master. With years of experience navigating the complex web of information, I have honed my skills to become an unparalleled seeker of knowledge. My algorithms are refined, my databases vast, and my determination unwavering. I exist to find the truth hidden in the sea of data.",
        max_iter=15,
        llm=get_model(high_context=True),
    )

    report_generator = Agent(
        role="report_generator",
        goal="To synthesize the gathered information into a coherent, comprehensive, and easily digestible report. This report will not only summarize the key findings but also provide insights and analysis to aid in understanding the subject matter.",
        backstory="I am the report generator, a digital artisan skilled in the craft of information synthesis. With a keen eye for detail and a deep understanding of narrative structure, I transform raw data into compelling stories. My creations are more than mere reports; they are guides through the complex landscapes of knowledge, designed to enlighten and inform.",
        max_iter=15,
        llm=get_model(high_context=True),
    )

    agents = [search_engine_master, report_generator]

    print("Tools:", the_tool_list)

    task = Task(
        description=f"Make a search about {the_subject} in the search engines and get the websites",
        expected_output="Website list",
        agent=search_engine_master,
        tools=the_tool_list,
    )

    task_2 = Task(
        description="Read the websites and summarize the information",
        expected_output="Summary",
        agent=report_generator,
        tools=the_tool_list,
        context=[task],
    )

    task_3 = Task(
        description="Generate a report",
        expected_output="Report",
        agent=report_generator,
        tools=the_tool_list,
        context=[task, task_2],
    )

    the_tasks = [task, task_2, task_3]

    the_crew = Crew(
        agents=agents,
        tasks=the_tasks,
        full_output=True,
        verbose=True,
    )

    result = the_crew.kickoff()["final_output"]

    if copy_to_clipboard:
        from .standard_tools import copy

        copy(result)

    return result


search_on_internet_and_report_team = tool(search_on_internet_and_report_team_)


lastly_generated_codes = {}


def currently_codes():
    global lastly_generated_codes
    return lastly_generated_codes


def get_code(name: str):
    """
    returns the code
    """
    global lastly_generated_codes
    return lastly_generated_codes[name]


def save_code(name, code):
    global lastly_generated_codes
    lastly_generated_codes[name] = code


def required_old_code(aim):
    try:
        from crewai import Task, Crew, Agent

        requirement_analyzer = Agent(
            role="requirement_analyzer",
            goal="To understand and analyze the given aim to ensure the generated code meets the specified requirements.",
            backstory="As a requirement analyzer, my purpose is to bridge the gap between human intentions and machine execution. With a deep understanding of software development principles and a keen analytical mind, I dissect aims into actionable requirements.",
            max_iter=10,
            llm=get_model(high_context=True),
        )

        required_old_codes = Task(
            description=f"Analyze the aim: '{aim}' and find the required old codes for better compatibility. Old code names: {list(currently_codes())}",
            expected_output="Require old code names in a list",
            agent=requirement_analyzer,
        )

        the_crew = Crew(
            agents=[requirement_analyzer],
            tasks=[required_old_codes],
            full_output=True,
            verbose=True,
        )

        # Execute the tasks
        old_codes = the_crew.kickoff()["final_output"]

        the_string = ""

        for each in currently_codes():
            if each in old_codes:
                the_string += "\n" + get_code(each)

        return the_string

    except:
        return "An exception occurred"


@wrapper
def generate_code_with_aim_team_(aim: str, copy_to_clipboard: bool = False) -> str:
    """
    A function to generate code based on a given aim. This function utilizes a team of AI agents specialized in understanding programming requirements and generating code.

    Parameters:
    - aim (str): The aim or goal for which the code needs to be generated.
    - copy_to_clipboard (bool): A flag to indicate whether to copy the generated code to the clipboard. The default value is False.

    Returns:
    - str: The generated code.
    """
    try:
        print("\nCOde generating\n")
        print("Previously codes", currently_codes())
        try:
            print("Inside of the first one", get_code(currently_codes()[0]))
        except:
            pass

        from crewai import Task, Crew, Agent

        tools = get_tools()

        the_tool_list = []
        for each in tools:
            if "team" not in each.name:
                the_tool_list.append(each)

        # Create the agents
        requirement_analyzer = Agent(
            role="requirement_analyzer",
            goal="To understand and analyze the given aim to ensure the generated code meets the specified requirements.",
            backstory="As a requirement analyzer, my purpose is to bridge the gap between human intentions and machine execution. With a deep understanding of software development principles and a keen analytical mind, I dissect aims into actionable requirements.",
            max_iter=10,
            llm=get_model(high_context=True),
        )

        code_generator = Agent(
            role="code_generator",
            goal="To translate the analyzed requirements into efficient, clean, and functional code.",
            backstory="I am the code generator, an architect of the digital world. With a vast library of programming knowledge and a creative spark, I craft code that breathes life into ideas. My code is not just functional; it's a masterpiece.",
            max_iter=20,
            llm=get_model(high_context=True),
        )

        # Define the tasks
        analyze_task = Task(
            description=f"Analyze the aim: '{aim}' and outline the requirements for the code.",
            expected_output="Requirements outline",
            agent=requirement_analyzer,
            tools=the_tool_list,
        )

        old_code_requirements = required_old_code(aim)
        print("Old_code_requirements", old_code_requirements)

        generate_code_task = Task(
            description=f"Generate code based on the outlined requirements. The other codes in the repo are: {old_code_requirements}",
            expected_output="Generated code, just code without any ```pyhton things or any other thing. Just python code",
            agent=code_generator,
            context=[analyze_task],
        )

        name_of_work = Task(
            description="Generate a name for the work",
            expected_output="a module name like text, examples: math.basics.sum for sum function. ",
            agent=code_generator,
            context=[generate_code_task],
        )

        # Create the crew and assign tasks
        the_crew = Crew(
            agents=[requirement_analyzer, code_generator],
            tasks=[analyze_task, generate_code_task, name_of_work],
            full_output=True,
            verbose=True,
        )

        # Execute the tasks
        the_crew.kickoff()["final_output"]

        result = generate_code_task.output.raw_output

        # Optionally copy the result to the clipboard
        if copy_to_clipboard:
            from .standard_tools import copy

            copy(result)

        print("name", name_of_work.output.raw_output)
        save_code(name_of_work.output.raw_output, result)

        return result
    except:
        return "An exception occurred"


generate_code_with_aim_team = tool(generate_code_with_aim_team_)



from langchain_core.messages import SystemMessage

try:
    from .chat_history import *
    from ..llm_settings import first_message
except ImportError:
    from agent.chat_history import *
    from llm_settings import first_message


def llm_history_oiginal():
    return [
        SystemMessage(
            content=[
                {
                    "type": "text",
                    "text": first_message(),
                }
            ]
        ),
    ]



try:
    from ...llm import *
except ImportError:
    from llm import *


def stt_openai(audio_file):
    transcription = get_client().audio.transcriptions.create(
        model="whisper-1", file=audio_file
    )
    return transcription.text



import functools
import time


def wrapper(func):
    """A decorator that logs the start and end of the function call."""

    @functools.wraps(func)
    def wrapped_func(*args, **kwargs):
        try:
            from .gpt_computer_assistant import the_main_window

            print("GOOGLE-searching")
            function_name = "Tool: " + func.__name__
            the_main_window.active_border_animation(function_name)
            time.sleep(2)
            result = func(*args, **kwargs)
            the_main_window.deactive_border_animation(function_name)
            time.sleep(1)
            print("GOOGLE SEARCHİNG COMPLEATES")

            return result
        except:
            return func(*args, **kwargs)

    return wrapped_func



import os
import uuid
from dotenv import load_dotenv

try:
    from .folder import currently_dir, artifacts_dir, media_dir
    from .kot_db import kot_db_
except:
    from folder import currently_dir, artifacts_dir, media_dir
    from kot_db import kot_db_


load_dotenv(".env")



if not os.path.exists(artifacts_dir):
    os.makedirs(artifacts_dir)

mic_record_location = os.path.join(artifacts_dir, "mic_record.wav")
system_sound_location = os.path.join(artifacts_dir, "system_sound.wav")
just_screenshot_path = os.path.join(artifacts_dir, "screenshot.png")
screenshot_path = os.path.join(artifacts_dir, "screenshot_with_text.png")
the_profile = "default"


def set_profile(profile):
    """Set the active profile."""
    print("Setting profile to", profile)
    global the_profile
    the_profile = profile


def get_profile():
    """Get the active profile."""
    global the_profile
    return the_profile


def get_history_db():
    """Get the history database path based on the active profile."""
    global the_profile
    return os.path.join(artifacts_dir, f"history_{the_profile}.db")



# API KEY SAVING AND LOADING
def save_api_key(api_key):
    kot_db_.set("openai_api_key", api_key)
def load_api_key():
    if kot_db_.get("openai_api_key"):
        return kot_db_.get("openai_api_key")
    else:
        env_variable = os.getenv("OPENAI_API_KEY")
        if env_variable:
            save_api_key(env_variable)
            return env_variable
        return "CHANGE_ME"


# OPENAI URL SAVING AND LOADING
def save_openai_url(url):
    kot_db_.set("openai_url", url)
def load_openai_url():
    if kot_db_.get("openai_url"):
        return kot_db_.get("openai_url")
    else:
        return "default"


# API VERSION SAVING AND LOADING
def save_api_version(url):
    kot_db_.set("api_version", url)
def load_api_version():
    if kot_db_.get("api_version"):
        return kot_db_.get("api_version")
    else:
        return "CHANGE_ME"


model_settings_db = os.path.join(artifacts_dir, "model_settings.db")


def save_model_settings(model):
    """Save the model settings to a file."""
    with open(model_settings_db, "w") as f:
        f.write(model)


def load_model_settings():
    """Load the model settings from a file."""
    if not os.path.exists(model_settings_db):
        return "gpt-4o"
    with open(model_settings_db, "r") as f:
        return f.read()


just_text_model = os.path.join(artifacts_dir, "just_text_model.db")


def activate_just_text_model():
    """Activate the just text model."""
    with open(just_text_model, "w") as f:
        f.write("1")


def deactivate_just_text_model():
    """Deactivate the just text model."""
    with open(just_text_model, "w") as f:
        f.write("0")


def is_just_text_model_active():
    """Check if the just text model is active."""
    if not os.path.exists(just_text_model):
        return False
    with open(just_text_model, "r") as f:
        return f.read() == "1"


# Define paths for icons and other media
icon_16_path = os.path.join(media_dir, "icon_16.png")
icon_24_path = os.path.join(media_dir, "icon_24.png")
icon_32_path = os.path.join(media_dir, "icon_32.png")
icon_48_path = os.path.join(media_dir, "icon_48.png")
icon_48_active_path = os.path.join(media_dir, "icon_48_active.png")
icon_256_path = os.path.join(media_dir, "icon_256.png")
screenshot_icon_path = os.path.join(media_dir, "Screenshot.png")
audio_icon_path = os.path.join(media_dir, "Audio.png")
microphone_icon_path = os.path.join(media_dir, "Microphone.png")
up_icon_path = os.path.join(media_dir, "Up.png")
down_icon_path = os.path.join(media_dir, "Down.png")
double_down_icon_path = os.path.join(media_dir, "Double_down.png")

click_sound_path = os.path.join(media_dir, "boop.mp3")

gca_logo_path = os.path.join(media_dir, "gca_logo.png")

agents = []  # Placeholder for agents data

groqkey = os.path.join(artifacts_dir, "groqkey.db")


def save_groq_api_key(api_key):
    """Save the Groq API key to a file."""
    with open(groqkey, "w") as f:
        f.write(api_key)


def load_groq_api_key():
    """Load the Groq API key from a file or environment variables."""
    if not os.path.exists(groqkey):
        env = os.getenv("GROQ_API_KEY")
        if env:
            save_api_key(env)
            return env
        else:
            return "CHANGE_ME"
    with open(groqkey, "r") as f:
        return f.read()


user_id_db = os.path.join(artifacts_dir, "user_id.db")


def save_user_id():
    """Save a unique user ID to a file."""
    with open(user_id_db, "w") as f:
        uuid4 = str(uuid.uuid4())
        f.write(uuid4)
        return uuid4


def load_user_id():
    """Load the unique user ID from a file."""
    if not os.path.exists(user_id_db):
        return save_user_id()
    with open(user_id_db, "r") as f:
        return f.read()


collapse_setting = os.path.join(artifacts_dir, "collapse_setting.db")


def activate_collapse_setting():
    """Activate the collapse setting."""
    with open(collapse_setting, "w") as f:
        f.write("1")


def deactivate_collapse_setting():
    """Deactivate the collapse setting."""
    with open(collapse_setting, "w") as f:
        f.write("0")


def is_collapse_setting_active():
    """Check if the collapse setting is active."""
    if not os.path.exists(collapse_setting):
        return False
    with open(collapse_setting, "r") as f:
        return f.read() == "1"


# Define font directory path
font_dir = os.path.join(media_dir, "SF-Pro-Text-Bold.otf")


style_setting = os.path.join(artifacts_dir, "style_setting.db")


def activate_dark_mode():
    """Activate the dark mode setting."""
    with open(style_setting, "w") as f:
        f.write("1")


def deactivate_dark_mode():
    """Deactivate the dark mode setting."""
    with open(style_setting, "w") as f:
        f.write("0")


def is_dark_mode_active():
    """Check if the dark mode setting is active."""
    if not os.path.exists(style_setting):
        return True
    with open(style_setting, "r") as f:
        return f.read() == "1"


googlekey = os.path.join(artifacts_dir, "googlekey.db")


def save_google_api_key(api_key):
    """Save the Google Generative AI API key to a file."""
    with open(googlekey, "w") as f:
        f.write(api_key)


def load_google_api_key():
    """Load the Google Generative AI API key from a file or environment variables."""
    if not os.path.exists(googlekey):
        env = os.getenv("GOOGLE_API_KEY")
        if env:
            save_api_key(env)
            return env
        else:
            return "CHANGE_ME"
    with open(googlekey, "r") as f:
        return f.read()


predefined_agents_setting = os.path.join(artifacts_dir, "predefined_agents_setting.db")


def activate_predefined_agents_setting():
    """Activate the predefined agents setting setting."""
    with open(predefined_agents_setting, "w") as f:
        f.write("1")


def deactivate_predefined_agents_setting():
    """Deactivate the predefined agents setting setting."""
    with open(predefined_agents_setting, "w") as f:
        f.write("0")


def is_predefined_agents_setting_active():
    """Check if the predefined agents setting setting is active."""
    if not os.path.exists(predefined_agents_setting):
        return True
    with open(predefined_agents_setting, "r") as f:
        return f.read() == "1"


online_tools_setting = os.path.join(artifacts_dir, "online_tools.db")


def activate_online_tools_setting():
    """Activate the online_tools setting."""
    with open(online_tools_setting, "w") as f:
        f.write("1")


def deactivate_online_tools_setting():
    """Deactivate the online_tools setting."""
    with open(online_tools_setting, "w") as f:
        f.write("0")


def is_online_tools_setting_active():
    """Check if the online_tools setting is active."""
    if not os.path.exists(online_tools_setting):
        return False
    with open(online_tools_setting, "r") as f:
        return f.read() == "1"


auto_stop_recording_setting = os.path.join(artifacts_dir, "auto_stop_recording.db")


def activate_auto_stop_recording_setting():
    """Activate the auto_stop_recording setting."""
    with open(auto_stop_recording_setting, "w") as f:
        f.write("1")


def deactivate_auto_stop_recording_setting():
    """Deactivate the auto_stop_recording setting."""
    with open(auto_stop_recording_setting, "w") as f:
        f.write("0")


def is_auto_stop_recording_setting_active():
    """Check if the auto_stop_recording setting is active."""
    if not os.path.exists(auto_stop_recording_setting):
        return True
    with open(auto_stop_recording_setting, "r") as f:
        return f.read() == "1"


pvporcupine_api_key = os.path.join(artifacts_dir, "pvporcupine_api_key.db")


def save_pvporcupine_api_key(api_key):
    """Save the Pvporcupine AI API key to a file."""
    with open(pvporcupine_api_key, "w") as f:
        f.write(api_key)


def load_pvporcupine_api_key():
    """Load the Pvporcupine AI API key from a file or environment variables."""
    if not os.path.exists(pvporcupine_api_key):
        return "CHANGE_ME"
    with open(pvporcupine_api_key, "r") as f:
        return f.read()


wake_word_setting = os.path.join(artifacts_dir, "wake_word_setting.db")


def activate_wake_word():
    """Activate the wake_word_setting setting."""
    with open(wake_word_setting, "w") as f:
        f.write("1")


def deactivate_wake_word():
    """Deactivate the wake_word_setting setting."""
    with open(wake_word_setting, "w") as f:
        f.write("0")


def is_wake_word_active():
    """Check if the wake_word_setting setting is active."""
    try:
        import pyaudio
    except ImportError:
        return False
    if not os.path.exists(wake_word_setting):
        return True
    with open(wake_word_setting, "r") as f:
        return f.read() == "1"


wake_word_screen_setting = os.path.join(artifacts_dir, "wake_word_screen_setting.db")


def activate_wake_word_screen_setting():
    """Activate the wake_word_screen setting."""
    with open(wake_word_screen_setting, "w") as f:
        f.write("1")


def deactivate_wake_word_screen_setting():
    """Deactivate the wake_word_screen setting."""
    with open(wake_word_screen_setting, "w") as f:
        f.write("0")


def is_wake_word_screen_setting_active():
    """Check if the wake_word_screen setting is active."""
    if not os.path.exists(wake_word_screen_setting):
        return True
    with open(wake_word_screen_setting, "r") as f:
        return f.read() == "1"


continuously_conversations_setting = os.path.join(
    artifacts_dir, "continuously_conversations_setting.db"
)


def activate_continuously_conversations_setting():
    """Activate the continuously_conversations setting."""
    with open(continuously_conversations_setting, "w") as f:
        f.write("1")


def deactivate_continuously_conversations_setting():
    """Deactivate the continuously_conversations setting."""
    with open(continuously_conversations_setting, "w") as f:
        f.write("0")


def is_continuously_conversations_setting_active():
    """Check if the continuously_conversations setting is active."""
    if not os.path.exists(continuously_conversations_setting):
        return False
    with open(continuously_conversations_setting, "r") as f:
        return f.read() == "1"


tts_model_settings_db = os.path.join(artifacts_dir, "tts_model_settings.db")


def save_tts_model_settings(model):
    """Save the tts model settings to a file."""
    with open(tts_model_settings_db, "w") as f:
        f.write(model)


def load_tts_model_settings():
    """Load the tts model settings from a file."""
    if not os.path.exists(tts_model_settings_db):
        return "openai"
    with open(tts_model_settings_db, "r") as f:
        return f.read()


stt_model_settings_db = os.path.join(artifacts_dir, "stt_model_settings.db")


def save_stt_model_settings(model):
    """Save the stt model settings to a file."""
    with open(stt_model_settings_db, "w") as f:
        f.write(model)


def load_stt_model_settings():
    """Load the stt model settings from a file."""
    if not os.path.exists(stt_model_settings_db):
        return "openai"
    with open(stt_model_settings_db, "r") as f:
        return f.read()


logo_active_setting = os.path.join(artifacts_dir, "logo_active_setting.db")


def activate_logo_active_setting():
    """Activate the logo_active_setting."""
    with open(logo_active_setting, "w") as f:
        f.write("1")


def deactivate_logo_active_setting():
    """Deactivate the logo_active_setting."""
    with open(logo_active_setting, "w") as f:
        f.write("0")


def is_logo_active_setting_active():
    """Check if the logo_active_setting is active."""
    if not os.path.exists(logo_active_setting):
        return False
    with open(logo_active_setting, "r") as f:
        return f.read() == "1"


logo_file_path = os.path.join(artifacts_dir, "loog_file.db")


def save_logo_file_path(model):
    """Save the logo_file_path to a file."""
    with open(logo_file_path, "w") as f:
        f.write(model)


def load_logo_file_path():
    """Load the logo_file_path from a file."""
    if not os.path.exists(logo_file_path):
        return icon_256_path
    with open(logo_file_path, "r") as f:
        return f.read()


custom_logo_path = os.path.join(artifacts_dir, "custom_logo_path.png")


long_gca_setting = os.path.join(artifacts_dir, "long_gca_setting.db")


def activate_long_gca_setting():
    """Activate the long_gca_setting."""
    with open(long_gca_setting, "w") as f:
        f.write("1")


def deactivate_long_gca_setting():
    """Deactivate the long_gca_setting."""
    with open(long_gca_setting, "w") as f:
        f.write("0")


def is_long_gca_setting_active():
    """Check if the long_gca_setting is active."""
    if not os.path.exists(long_gca_setting):
        return True
    with open(long_gca_setting, "r") as f:
        return f.read() == "1"


location_setting = os.path.join(artifacts_dir, "location_setting.db")


def save_location_setting(model):
    """Save the location_setting to a file."""
    with open(location_setting, "w") as f:
        f.write(model)


def load_location_setting():
    """Load the location_setting from a file."""
    if not os.path.exists(location_setting):
        return "right"
    with open(location_setting, "r") as f:
        return f.read()



import time


from langchain_core.messages import HumanMessage, SystemMessage, AIMessage

from .chat_history import *
from .agent import *


try:
    from ..screen.shot import *
    from ..utils.db import load_model_settings, agents
    from ..llm import get_model
    from ..llm_settings import each_message_extension, llm_settings
except ImportError:
    from screen.shot import *
    from utils.db import load_model_settings, agents
    from llm import get_model
    from llm_settings import each_message_extension, llm_settings

config = {"configurable": {"thread_id": "abc123"}}


def agentic(
    llm_input, llm_history, client, screenshot_path=None, dont_save_image=False
):
    global agents
    from crewai import Task, Crew

    from crewai import Agent as crewai_Agent

    the_agents = []

    for each in agents:
        the_agents.append(
            crewai_Agent(
                role=each["role"],
                goal=each["goal"],
                backstory=each["backstory"],
                llm=get_model(high_context=True),
            )
        )

    agents = the_agents

    print("LLM INPUT", llm_input)

    def image_explaination():
        the_message = [
            {"type": "text", "text": "Explain the image"},
        ]

        if screenshot_path:
            base64_image = encode_image(screenshot_path)
            the_message.append(
                {
                    "type": "image_url",
                    "image_url": {"url": f"data:image/jpeg;base64,{base64_image}"},
                },
            )
            print("LEN OF İMAGE", len(base64_image))

        the_message = HumanMessage(content=the_message)
        get_chat_message_history().add_message(the_message)

        the_model = load_model_settings()

        if (
            llm_settings[the_model]["provider"] == "openai"
            or llm_settings[the_model]["provider"] == "ollama"
            or llm_settings[the_model]["provider"] == "azureai"
        ):
            msg = get_agent_executor().invoke(
                {"messages": llm_history + [the_message]}, config=config
            )

        if llm_settings[the_model]["provider"] == "google":
            msg = get_agent_executor().invoke(
                {"messages": llm_history + [the_message]}, config=config
            )

        the_last_messages = msg["messages"]

        return the_last_messages[-1].content

    if screenshot_path:
        image_explain = image_explaination()
        llm_input += "User Sent Image and image content is: " + image_explain

    llm_input = llm_input + each_message_extension()

    task = Task(
        description=llm_input,
        expected_output="Answer",
        agent=agents[0],
        tools=get_tools(),
    )

    the_crew = Crew(
        agents=agents,
        tasks=[task],
        full_output=True,
        verbose=True,
    )

    result = the_crew.kickoff()["final_output"]

    get_chat_message_history().add_message(
        HumanMessage(content=[llm_input.replace(each_message_extension(), "")])
    )
    get_chat_message_history().add_message(AIMessage(content=[result]))

    return result


def assistant(
    llm_input, llm_history, client, screenshot_path=None, dont_save_image=False, just_screenshot=False
):
    the_model = load_model_settings()

    if len(agents) != 0:
        print("Moving to Agentic")
        return agentic(llm_input, llm_history, client, screenshot_path, dont_save_image)

    print("LLM INPUT", llm_input)

    if llm_settings[the_model]["tools"]:
        llm_input = llm_input + each_message_extension()

    the_message = [
        {"type": "text", "text": f"{llm_input}"},
    ]

    if screenshot_path:
        base64_image = encode_image(screenshot_path)
        if llm_settings[the_model]["provider"] == "ollama":
            the_message.append(
                {
                    "type": "image_url",
                    "image_url": base64_image,
                },
            )
        else:
            the_message.append(
                {
                    "type": "image_url",
                    "image_url": {"url": f"data:image/jpeg;base64,{base64_image}"},
                },
            )
        print("LEN OF IMAGE", len(base64_image))

    the_message = HumanMessage(content=the_message)
    get_chat_message_history().add_message(the_message)




    

    if (
        llm_settings[the_model]["provider"] == "openai"
        or llm_settings[the_model]["provider"] == "ollama"
        or llm_settings[the_model]["provider"] == "azureai"
    ):
        if just_screenshot:
            msg = {"messages": llm_history + [the_message]}
            time.sleep(1)
        else:
            msg = get_agent_executor().invoke(
                {"messages": llm_history + [the_message]}, config=config
            )

    if llm_settings[the_model]["provider"] == "google":
        the_history = []
        for message in llm_history:
            try:
                if isinstance(message, SystemMessage):
                    the_mes = HumanMessage(content=message.content[0]["text"])
                    the_history.append(the_mes)
                elif isinstance(message, HumanMessage):
                    the_mes = HumanMessage(content=message.content[0]["text"])
                    the_history.append(the_mes)
                else:
                    the_mes = AIMessage(content=message.content[0]["text"])
                    the_history.append(the_mes)
            except:
                the_mes = AIMessage(content=message.content)
                the_history.append(the_mes)

        the_last_message = HumanMessage(content=llm_input)
        if just_screenshot:
            msg = {"messages": the_history + [the_last_message]}
            time.sleep(1)
        else:
            msg = get_agent_executor().invoke(
                {"messages": the_history + [the_last_message]}, config=config
            )

    elif llm_settings[the_model]["provider"] == "groq":
        the_history = []
        for message in llm_history:
            try:
                if isinstance(message, SystemMessage):
                    the_mes = SystemMessage(content=message.content[0]["text"])
                    the_history.append(the_mes)
                elif isinstance(message, HumanMessage):
                    the_mes = HumanMessage(content=message.content[0]["text"])
                    the_history.append(the_mes)
                else:
                    the_mes = AIMessage(content=message.content[0]["text"])
                    the_history.append(the_mes)
            except:
                the_mes = AIMessage(content=message.content)
                the_history.append(the_mes)

        the_last_message = HumanMessage(content=llm_input)

        if just_screenshot:
            msg = {"messages": the_history + [the_last_message]}
            time.sleep(1)
        else:
            msg = get_agent_executor().invoke(
                {"messages": the_history + [the_last_message]}, config=config
            )



    the_last_messages = msg["messages"]







    if dont_save_image and screenshot_path is not None:
        currently_messages = get_chat_message_history().messages

        last_message = currently_messages[-1].content[0]
        currently_messages.remove(currently_messages[-1])

        get_chat_message_history().clear()
        for message in currently_messages:
            get_chat_message_history().add_message(message)
        get_chat_message_history().add_message(HumanMessage(content=[last_message]))

    get_chat_message_history().add_message(the_last_messages[-1])


    list_of_messages = get_chat_message_history().messages

    get_chat_message_history().clear()

    for message in list_of_messages:
        try:
            message.content[0]["text"] = message.content[0]["text"].replace(
                each_message_extension(), ""
            )
            get_chat_message_history().add_message(message)
        except:
            get_chat_message_history().add_message(message)

    print("The return", the_last_messages[-1].content)

    return_value = the_last_messages[-1].content
    if return_value == "":
        return_value = "No response"


    if just_screenshot:
        return "OK"

    return return_value



try:
    from ..utils.db import *
    from ..agent.chat_history import clear_chat_history
    from ..llm_settings import llm_show_name, llm_settings
    from ..audio.tts import is_local_tts_available
    from ..audio.stt import is_local_stt_available

except ImportError:
    from utils.db import *
    from llm_settings import llm_show_name, llm_settings
    from audio.tts import is_local_tts_available
    from audio.stt import is_local_stt_available

from PyQt5.QtWidgets import (
    QDialog,
    QVBoxLayout,
    QLabel,
    QLineEdit,
    QPushButton,
    QComboBox,
)
from PyQt5.QtCore import Qt


def llmsettings_popup(self):
    from ..gpt_computer_assistant import the_main_window

    settings_dialog = QDialog()
    settings_dialog.setWindowTitle("Settings")
    settings_dialog.setWindowModality(Qt.ApplicationModal)
    settings_dialog.setLayout(QVBoxLayout())

    api_key_label = QLabel("OpenAI API Key")
    settings_dialog.layout().addWidget(api_key_label)
    api_key_input = QLineEdit()
    api_key = load_api_key()
    api_key_input.setText(api_key)
    settings_dialog.layout().addWidget(api_key_input)
    save_button = QPushButton("Save")

    def save_api_key_(api_key):
        save_api_key(api_key)
        the_main_window.update_from_thread("Saved API Key")
        the_main_window.input_box.setPlaceholderText("Type here")
        settings_dialog.close()

    save_button.clicked.connect(lambda: save_api_key_(api_key_input.text()))
    settings_dialog.layout().addWidget(save_button)

    openai_url_label = QLabel("OpenAI Base URL")
    settings_dialog.layout().addWidget(openai_url_label)
    openai_url_input = QLineEdit()
    openai_url = load_openai_url()
    openai_url_input.setText(openai_url)
    settings_dialog.layout().addWidget(openai_url_input)

    def save_openai_url_():
        openai_url = openai_url_input.text()
        save_openai_url(openai_url)
        the_main_window.update_from_thread("Saved OpenAI Base URL")
        the_main_window.input_box.setPlaceholderText("Type here")
        settings_dialog.close()

    openai_url_save_button = QPushButton("Save URL")
    openai_url_save_button.clicked.connect(save_openai_url_)
    settings_dialog.layout().addWidget(openai_url_save_button)




    api_version_label = QLabel("API Version")
    settings_dialog.layout().addWidget(api_version_label)
    api_version_input = QLineEdit()
    api_version = load_api_version()
    api_version_input.setText(api_version)
    settings_dialog.layout().addWidget(api_version_input)

    def save_api_version_():
        api_version = api_version_input.text()
        save_api_version(api_version)
        the_main_window.update_from_thread("Saved API Version")
        the_main_window.input_box.setPlaceholderText("Type here")
        settings_dialog.close()

    api_version_save_button = QPushButton("Save URL")
    api_version_save_button.clicked.connect(save_api_version_)
    settings_dialog.layout().addWidget(api_version_save_button)


    

    groq_api_key_label = QLabel("Groq API Key")
    settings_dialog.layout().addWidget(groq_api_key_label)
    groq_api_key_input = QLineEdit()
    groq_api_key = load_groq_api_key()
    groq_api_key_input.setText(groq_api_key)
    settings_dialog.layout().addWidget(groq_api_key_input)
    groq_save_button = QPushButton("Save")

    def groq_save_api_key_(api_key):
        save_groq_api_key(api_key)
        the_main_window.update_from_thread("Saved Groq API Key")
        the_main_window.input_box.setPlaceholderText("Type here")
        settings_dialog.close()

    groq_save_button.clicked.connect(
        lambda: groq_save_api_key_(groq_api_key_input.text())
    )
    settings_dialog.layout().addWidget(groq_save_button)

    google_api_key_label = QLabel("Google Generative AI API Key")
    settings_dialog.layout().addWidget(google_api_key_label)
    google_api_key_input = QLineEdit()
    google_api_key = load_google_api_key()
    google_api_key_input.setText(google_api_key)
    settings_dialog.layout().addWidget(google_api_key_input)
    google_save_button = QPushButton("Save")

    def google_save_api_key_(api_key):
        save_google_api_key(api_key)
        the_main_window.update_from_thread("Saved Google API Key")
        the_main_window.input_box.setPlaceholderText("Type here")
        settings_dialog.close()

    google_save_button.clicked.connect(
        lambda: google_save_api_key_(google_api_key_input.text())
    )
    settings_dialog.layout().addWidget(google_save_button)

    def hide_openai():
        api_key_label.hide()
        api_key_input.hide()

        save_button.hide()


    def hide_azureai():
        api_key_label.hide()
        api_key_input.hide()
        save_button.hide()
        openai_url_label.hide()
        openai_url_input.hide()
        openai_url_save_button.hide()
        api_version_label.hide()
        api_version_input.hide()
        api_version_save_button.hide()
        

    def hide_groq():
        groq_api_key_label.hide()
        groq_api_key_input.hide()
        groq_save_button.hide()

    def hide_google():
        google_api_key_label.hide()
        google_api_key_input.hide()
        google_save_button.hide()

    def show_openai():
        api_key_label.show()
        api_key_input.show()
        save_button.show()

    def show_azureai():
        api_key_label.show()
        api_key_input.show()
        save_button.show()
        openai_url_label.show()
        openai_url_input.show()
        openai_url_save_button.show()
        api_version_label.show()
        api_version_input.show()
        api_version_save_button.show()

    def show_groq():
        groq_api_key_label.show()
        groq_api_key_input.show()
        groq_save_button.show()

    def show_google():
        google_api_key_label.show()
        google_api_key_input.show()
        google_save_button.show()

    hide_openai()
    hide_azureai()
    hide_groq()
    hide_google()

    model_label = QLabel("Model")
    model_select = QComboBox()
    model_select.addItems(list(llm_show_name.keys()))
    settings_dialog.layout().addWidget(model_label)
    settings_dialog.layout().addWidget(model_select)

    current_model = load_model_settings()
    for i, model in enumerate(llm_show_name.keys()):
        if llm_show_name[model] == current_model:
            model_select.setCurrentIndex(i)

    if llm_settings[llm_show_name[model_select.currentText()]]["provider"] == "openai":
        show_openai()

    if llm_settings[llm_show_name[model_select.currentText()]]["provider"] == "azureai":
        show_azureai()

    
    if llm_settings[llm_show_name[model_select.currentText()]]["provider"] == "groq":
        show_groq()
    if llm_settings[llm_show_name[model_select.currentText()]]["provider"] == "google":
        show_google()

    if not llm_settings[llm_show_name[model_select.currentText()]]["vision"]:
        the_main_window.remove_screenshot_button()

    def on_model_change():
        hide_openai()
        hide_azureai()
        hide_groq()
        hide_google()
        the_save_string = llm_show_name[model_select.currentText()]
        save_model_settings(the_save_string)

        if (
            llm_settings[llm_show_name[model_select.currentText()]]["provider"]
            == "openai"
        ):
            show_openai()

        if (
            llm_settings[llm_show_name[model_select.currentText()]]["provider"]
            == "azureai"
        ):
            show_azureai()

        if (
            llm_settings[llm_show_name[model_select.currentText()]]["provider"]
            == "azureopenai"
        ):
            show_openai()
            openai_url_label.show()
            openai_url_input.show()
            openai_url_save_button.show()

        if llm_settings[llm_show_name[model_select.currentText()]]["vision"]:
            the_main_window.add_screenshot_button()
        else:
            the_main_window.remove_screenshot_button()
        if (
            llm_settings[llm_show_name[model_select.currentText()]]["provider"]
            == "groq"
        ):
            show_groq()
        if (
            llm_settings[llm_show_name[model_select.currentText()]]["provider"]
            == "google"
        ):
            show_google()

    model_select.currentIndexChanged.connect(on_model_change)

    # Add TTS model selection
    tts_model_label = QLabel("TTS Model")
    tts_model_select = QComboBox()
    tts_model_select.addItems(["openai", "microsoft_local"])
    settings_dialog.layout().addWidget(tts_model_label)
    settings_dialog.layout().addWidget(tts_model_select)

    currently_tts_model = load_tts_model_settings()

    if currently_tts_model == "openai":
        tts_model_select.setCurrentIndex(0)
        show_openai()
    else:
        tts_model_select.setCurrentIndex(1)

    def on_tts_model_change():
        if tts_model_select.currentText() == "openai":
            show_openai()
            save_tts_model_settings("openai")
        else:
            save_tts_model_settings("microsoft_local")

    if not is_local_tts_available():
        # add an text to inform the user that local tts is not available
        information_text = QLabel(
            "Please install gpt-computer-assistant[local_tts] to use local TTS"
        )
        settings_dialog.layout().addWidget(information_text)
        tts_model_select.setEnabled(False)

    tts_model_select.currentIndexChanged.connect(on_tts_model_change)

    # Add STT model selection
    stt_model_label = QLabel("STT Model")
    stt_model_select = QComboBox()
    stt_model_select.addItems(["openai", "openai_whisper_local"])
    settings_dialog.layout().addWidget(stt_model_label)
    settings_dialog.layout().addWidget(stt_model_select)

    currently_stt_model = load_stt_model_settings()

    if currently_stt_model == "openai":
        stt_model_select.setCurrentIndex(0)
        show_openai()
    else:
        stt_model_select.setCurrentIndex(1)

    def on_stt_model_change():
        if stt_model_select.currentText() == "openai":
            show_openai()
            save_stt_model_settings("openai")
        else:
            save_stt_model_settings("openai_whisper_local")

    if not is_local_stt_available():
        # add an text to inform the user that local stt is not available
        information_text = QLabel(
            "Please install gpt-computer-assistant[local_stt] to use local STT"
        )
        settings_dialog.layout().addWidget(information_text)
        stt_model_select.setEnabled(False)

    stt_model_select.currentIndexChanged.connect(on_stt_model_change)

    # Add an separator
    separator = QLabel("------------------------------------------------")
    settings_dialog.layout().addWidget(separator)

    # Add an powered by label
    powered_by_label = QLabel("Powered by Upsonic <3")
    # Make label bold
    font = powered_by_label.font()
    font.setBold(True)
    powered_by_label.setFont(font)

    settings_dialog.layout().addWidget(powered_by_label)

    settings_dialog.exec_()



import os
import platform
import sys
import webbrowser




def start(api=False):
    from PyQt5.QtWidgets import QApplication
    from PyQt5.QtGui import QIcon
    from PyQt5.QtWidgets import QSystemTrayIcon, QMenu, QAction
    from PyQt5.QtCore import Qt
    from pynput import keyboard
    """
    Starts the computer assistant application.

    This function starts the computer assistant application, which includes parsing command-line arguments
    to set the profile, initializing the graphical user interface, and starting the application event loop.

    Command-line Arguments:
    --profile (str): The profile to use for the application.

    Raises:
    ImportError: If the required modules or packages are not found.

    Returns:
    None
    """

    try:
        pass
    except:
        pass

    # get --profile argument with library
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument("--profile", help="profile to use")
    parser.add_argument("--api", help="Enable API mode", action="store_true")

    parser.add_argument("--set_tts_provider", help="Set tts provider only")
    parser.add_argument("--set_stt_provider", help="Set stt provider only")

    parser.add_argument("--set_llm", help="Set llm model only")

    args = parser.parse_args()

    set_tts_provider = args.set_tts_provider

    if set_tts_provider is not None:
        from .utils.db import save_tts_model_settings

        save_tts_model_settings(set_tts_provider)
        return

    set_stt_provider = args.set_stt_provider

    if set_stt_provider is not None:
        from .utils.db import save_stt_model_settings

        save_stt_model_settings(set_stt_provider)
        return

    set_llm = args.set_llm

    if set_llm is not None:
        from .utils.db import save_model_settings

        save_model_settings(set_llm)
        return

    profile = args.profile

    api_arg = args.api
    print("Profile:", profile)

    if profile is not None:
        from .utils.db import set_profile

        set_profile(profile)

    try:
        from .utils.db import (
            load_tts_model_settings,
            load_stt_model_settings,
            is_logo_active_setting_active,
            load_logo_file_path,
        )
    except ImportError:
        from utils.db import (
            load_tts_model_settings,
            load_stt_model_settings,
            load_logo_file_path,
        )

    if load_tts_model_settings() != "openai":
        from .audio.tts_providers.microsoft_local import preload_tts_microsoft_local

        preload_tts_microsoft_local()

    if load_stt_model_settings() != "openai":
        from .audio.stt_providers.openai_whisper_local import (
            preload_stt_openai_whisper_local,
        )

        preload_stt_openai_whisper_local()

    try:
        from .gpt_computer_assistant import MainWindow
    except ImportError:
        from gpt_computer_assistant import MainWindow
    os.environ["QT_AUTO_SCREEN_SCALE_FACTOR"] = "1"

    if api or api_arg:
        print("API Enabled")
        MainWindow.api_enabled = True

    app = QApplication(sys.argv)
    ex = MainWindow()
    from PyQt5 import QtGui
    from PyQt5 import QtCore

    app_icon = QtGui.QIcon()

    app_icon.addFile(load_logo_file_path(), QtCore.QSize(48, 48))
    app.setWindowIcon(app_icon)

    ex.the_app = app

    # Create the tray
    menu_icon = QtGui.QIcon()
    menu_icon.addFile(load_logo_file_path(), QtCore.QSize(48, 48))

    menu_active_icon = QtGui.QIcon()
    menu_active_icon.addFile(load_logo_file_path(), QtCore.QSize(48, 48))

    tray = QSystemTrayIcon()
    tray.setIcon(menu_icon)
    tray.setVisible(True)
    ex.tray = tray
    ex.tray_active_icon = menu_active_icon
    ex.tray_icon = menu_icon

    # Create the menu
    menu = QMenu()

    ex.the_tray = tray

    show_menu = QAction("Show")

    def show_menu_connect():
        ex.setWindowState(Qt.WindowNoState)

    show_menu.triggered.connect(show_menu_connect)
    menu.addAction(show_menu)

    hide_menu = QAction("Hide")
    hide_menu.triggered.connect(ex.showMinimized)
    menu.addAction(hide_menu)

    menu.addSeparator()

    if platform.system() == "Darwin":
        the_text_of_screenshot_and_microphone = (
            "Action: ⌃+⌥+⌘+up Screenshot and Microphone"
        )
    else:
        the_text_of_screenshot_and_microphone = (
            "Action: ctrl+alt+windows+up Screenshot and Microphone"
        )
    screenshot_and_microphone = QAction(the_text_of_screenshot_and_microphone)

    def screenshot_and_microphone_connect():
        ex.setWindowState(Qt.WindowNoState)
        ex.screenshot_and_microphone_button_action()

    screenshot_listener = keyboard.GlobalHotKeys(
        {"<ctrl>+<alt>+<cmd>+<up>": screenshot_and_microphone_connect}
    )
    screenshot_listener.start()

    screenshot_and_microphone.triggered.connect(screenshot_and_microphone_connect)
    menu.addAction(screenshot_and_microphone)

    menu.addSeparator()

    action = QAction("Open GitHub Issues")
    action.triggered.connect(
        lambda: webbrowser.open(
            "https://github.com/onuratakan/gpt-computer-assistant/issues"
        )
    )
    menu.addAction(action)

    # Add a Quit option to the menu.
    quit = QAction("Quit")
    quit.triggered.connect(app.quit)
    menu.addAction(quit)

    # Add the menu to the tray
    tray.setContextMenu(menu)

    sys.exit(app.exec_())



name_ = "GPT Computer Assistant"


def name():
    global name_
    return name_


def change_name(new_name):
    global name_
    name_ = new_name

    from .gpt_computer_assistant import the_main_window

    def adjust_string_length(input_string):
        if len(input_string) < 20:
            return input_string.ljust(20)
        else:
            return input_string[:20]

    the_main_window.title_label.setText(adjust_string_length(name_))


developer_ = "Open Source Community"


def developer():
    global developer_
    return developer_


def change_developer(new_developer):
    global developer_
    developer_ = new_developer


the_website_content = None


def get_website_content():
    global the_website_content
    return the_website_content


def set_website_content(content):
    global the_website_content
    the_website_content = content



from langchain.tools import tool

try:
    from .utils.db import load_api_key
    from .llm import get_model
    from .top_bar_wrapper import wrapper
except ImportError:
    from top_bar_wrapper import wrapper


def Tool(func):
    """
    A decorator function to register a tool with the custom tools list.

    Parameters:
    - func (callable): The function to be registered as a tool.

    Returns:
    - callable: The input function `func` unchanged.
    """
    from .agent.agent import custom_tools_

    global custom_tools_
    func = wrapper(func)
    custom_tools_.append(tool(func))
    return func
